import { Component, ViewChild, OnInit, OnDestroy } from '@angular/core';
import { DivContentFormComponent } from "./div-help-content-form.component";
import { Subscription } from "rxjs/Subscription";
import { HelpContentService } from "../../services/help-content.service";
import { DivHelpContent } from "../../domain/div-help-content";
import { ActivatedRoute, Router } from "@angular/router";
import { EnvProperties } from '../../openaireLibrary/utils/properties/env-properties';

@Component({
    selector: 'edit-div-help-content',
    templateUrl: 'edit-div-help-content.component.html',
})

export class EditDivHelpContentComponent implements OnInit, OnDestroy{

    @ViewChild(DivContentFormComponent)
    public formComponent : DivContentFormComponent;

    public communityPid: string;
    public pageId: string;

    private sub: Subscription;

    private divHelpContent: DivHelpContent;

    public properties:EnvProperties = null;

    public showLoading: boolean = true;
    public errorMessage: string = '';
    public updateErrorMessage: string = '';

    constructor(
        private route: ActivatedRoute,
        private router: Router,
        private _helpContentService: HelpContentService) {}

    ngOnInit() {
      this.route.data
        .subscribe((data: { envSpecific: EnvProperties }) => {
           this.properties = data.envSpecific;
           this.sub = this.route.queryParams.subscribe(params => {
            //let id = params['id'];
            let divContentId = params['classContentId'];
            this.communityPid = params['communityId'];
            this.pageId = params['pageId'];

            this.getDivHelpContent(divContentId);
          });
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }

    handleError(message: string, error) {
        this.errorMessage = message;
        console.log('Server responded: ' + error);
        this.showLoading = false;
    }

    handleUpdateError(message: string, error) {
        this.updateErrorMessage = message;
        console.log('Server responded: ' + error);
        this.showLoading = false;
    }

    private getDivHelpContent(divContentId: string) {
      this.showLoading = true;
      this.errorMessage = "";
      this.updateErrorMessage = "";

      this._helpContentService.getDivHelpContent(divContentId, this.properties.adminToolsAPIURL).subscribe(
          divHelpContent => {
            this.updateForm(divHelpContent);
            this.showLoading = false;
          },
          error => this.handleError('System error retrieving class help content', error));
    }

    getDivId(divId: string) {
        this.showLoading = true;

        this._helpContentService.getDivIdFull(divId, this.properties.adminToolsAPIURL).subscribe(
          div => {
            this.formComponent.selectedDiv = div;

            if(!this.pageId) {
              this.formComponent.getDivs("");
            }
            
            this.showLoading = false;
          },
          error => this.handleError('System error retrieving class', error)
       );
    }

    private updateForm(divHelpContent : DivHelpContent) {
        this.divHelpContent = divHelpContent;

        this.getDivId(divHelpContent.divId as string);

        this.formComponent.myForm.patchValue((divHelpContent));
    }

    private saveCustom() {
        if(this.formComponent.myForm.valid) {
            this.showLoading = true;
            this.updateErrorMessage = "";

            let divHelpContent : DivHelpContent = this.formComponent.myForm.value;
            console.info(divHelpContent);

            this._helpContentService.insertOrUpdateDivHelpContent(divHelpContent, this.properties.adminToolsAPIURL).subscribe(
                _ => {
                  if(this.pageId) {
                    this.router.navigate( ['/classContents/'], { queryParams: { "communityId": this.communityPid, "pageId": this.pageId  } } );
                  } else {
                    this.router.navigate(['/classContents'], { queryParams: { "communityId": this.communityPid } } );
                  }
                  this.showLoading = false;
                },
                err => this.handleUpdateError('System error updating class content', err)
            );
        } else {
            this.errorMessage = "Please fill all required fields";
        }
    }

    private cancelCustom() {
      this.errorMessage = "";
      this.updateErrorMessage = "";

      if(this.pageId) {
        this.router.navigate( ['/classContents/'], { queryParams: { "communityId": this.communityPid, "pageId": this.pageId  } } );
      } else {
        this.router.navigate(['/classContents'], { queryParams: { "communityId": this.communityPid } } );
      }
    }
}
